@@ -1,4 +1,4 @@
-#     Copyright 2023, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2024, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -20,7 +20,7 @@
 """
 
 version_string = """\
-Nuitka V2.4.10
+Nuitka V2.5
 Copyright (C) 2024 Kay Hayen."""
 
 
