@@ -1,20 +1,6 @@
-#     Copyright 2023, Kay Hayen, mailto:kay.hayen@gmail.com
-#
-#     Part of "Nuitka", an optimizing Python compiler that is compatible and
-#     integrates with CPython, but also works on its own.
-#
-#     Licensed under the Apache License, Version 2.0 (the "License");
-#     you may not use this file except in compliance with the License.
-#     You may obtain a copy of the License at
-#
-#        http://www.apache.org/licenses/LICENSE-2.0
-#
-#     Unless required by applicable law or agreed to in writing, software
-#     distributed under the License is distributed on an "AS IS" BASIS,
-#     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-#     See the License for the specific language governing permissions and
-#     limitations under the License.
-#
+#     Copyright 2024, Kay Hayen, mailto:kay.hayen@gmail.com find license text at end of file
+
+
 """ Module/Package nodes
 
 The top of the tree. Packages are also modules. Modules are what hold a program
@@ -34,6 +20,7 @@ from nuitka.PythonVersions import python_version
 from nuitka.SourceCodeReferences import fromFilename
 from nuitka.tree.SourceHandling import parsePyIFile, readSourceCodeFromFilename
 from nuitka.utils.CStrings import encodePythonIdentifierToC
+from nuitka.utils.Importing import getPackageDirFilename
 from nuitka.utils.ModuleNames import ModuleName
 
 from .ChildrenHavingMixins import (
@@ -57,6 +44,7 @@ class PythonModuleBase(NodeBase):
 
     def __init__(self, module_name, reason, source_ref):
         assert type(module_name) is ModuleName, module_name
+        assert module_name != "", source_ref
 
         NodeBase.__init__(self, source_ref=source_ref)
 
@@ -144,7 +132,7 @@ class PythonModuleBase(NodeBase):
             )
 
     def getCodeName(self):
-        # Abstract method, pylint: disable=no-self-use
+        # Virtual method, pylint: disable=no-self-use
         return None
 
     def getCompileTimeFilename(self):
@@ -205,8 +193,19 @@ class PythonModuleBase(NodeBase):
 
                 result = os.path.join(os.path.basename(current), result)
 
+            # Avoid unnecessary unicode path values
+            if str is not bytes:
+                try:
+                    result = str(result)
+                except UnicodeDecodeError:
+                    pass
+
             return result
 
+    @staticmethod
+    def isExtensionModulePackage():
+        return False
+
 
 class CompiledPythonModule(
     ModuleChildrenHavingBodyOptionalStatementsOrNoneFunctionsTupleMixin,
@@ -456,7 +455,10 @@ class CompiledPythonModule(
         # For code name of modules, we need to translate to C identifiers,
         # removing characters illegal for that.
 
-        return encodePythonIdentifierToC(self.getFullName())
+        if self.code_name is None:
+            self.code_name = encodePythonIdentifierToC(self.module_name)
+
+        return self.code_name
 
     @staticmethod
     def getChildQualname(function_name):
@@ -546,9 +548,11 @@ class CompiledPythonModule(
 
         self.trace_collection = TraceCollectionModule(
             self,
-            very_trusted_module_variables=old_collection.getVeryTrustedModuleVariables()
-            if old_collection is not None
-            else {},
+            very_trusted_module_variables=(
+                old_collection.getVeryTrustedModuleVariables()
+                if old_collection is not None
+                else {}
+            ),
         )
 
         module_body = self.subnode_body
@@ -954,7 +958,7 @@ class PythonMainModule(CompiledPythonModule):
             )
 
             # Technically required, but not found must not happen
-            assert finding != "not-found"
+            assert finding != "not-found", early_module_name
 
             self.standard_library_modules.add(
                 makeModuleUsageAttempt(
@@ -972,13 +976,14 @@ class PythonMainModule(CompiledPythonModule):
 class PythonExtensionModule(PythonModuleBase):
     kind = "PYTHON_EXTENSION_MODULE"
 
-    __slots__ = ("used_modules", "technical")
+    __slots__ = ("used_modules", "module_filename", "technical", "is_package")
 
     avoid_duplicates = set()
 
     def __init__(
         self,
         module_name,
+        module_filename,
         reason,
         technical,
         source_ref,
@@ -1006,11 +1011,22 @@ class PythonExtensionModule(PythonModuleBase):
 
         self.used_modules = None
 
+        if os.path.isdir(module_filename):
+            module_filename = getPackageDirFilename(module_filename)
+            self.is_package = True
+        else:
+            self.is_package = False
+
+        self.module_filename = module_filename
+
+    def isExtensionModulePackage(self):
+        return self.is_package
+
     def finalize(self):
         del self.used_modules
 
     def getFilename(self):
-        return self.source_ref.getFilename()
+        return self.module_filename
 
     @staticmethod
     def startTraversal():
@@ -1020,24 +1036,31 @@ class PythonExtensionModule(PythonModuleBase):
         """Must be present as it's used in CPython library initialization."""
         return self.technical
 
-    def getPyIFilename(self):
+    def _getPyIFilename(self):
         """Get Python type description filename."""
 
         path = self.getFilename()
         filename = os.path.basename(path)
         dirname = os.path.dirname(path)
 
-        return os.path.join(dirname, filename.split(".")[0]) + ".pyi"
+        for suffix in (".pyi", ".py"):
+            candidate = os.path.join(dirname, filename.split(".")[0]) + suffix
+
+            if os.path.exists(candidate):
+                return candidate
+
+        return None
 
     def _readPyIFile(self):
         """Read the .pyi file if present and scan for dependencies."""
 
         if self.used_modules is None:
-            pyi_filename = self.getPyIFilename()
+            pyi_filename = self._getPyIFilename()
 
-            if os.path.exists(pyi_filename):
+            if pyi_filename is not None:
                 pyi_deps = parsePyIFile(
-                    module_name=self.getFullName(), pyi_filename=pyi_filename
+                    module_name=self.getFullName(),
+                    pyi_filename=pyi_filename,
                 )
 
                 # These are not to be taken serious.
@@ -1077,3 +1100,19 @@ class PythonExtensionModule(PythonModuleBase):
 
     def getParentModule(self):
         return self
+
+
+#     Part of "Nuitka", an optimizing Python compiler that is compatible and
+#     integrates with CPython, but also works on its own.
+#
+#     Licensed under the Apache License, Version 2.0 (the "License");
+#     you may not use this file except in compliance with the License.
+#     You may obtain a copy of the License at
+#
+#        http://www.apache.org/licenses/LICENSE-2.0
+#
+#     Unless required by applicable law or agreed to in writing, software
+#     distributed under the License is distributed on an "AS IS" BASIS,
+#     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+#     See the License for the specific language governing permissions and
+#     limitations under the License.
